@@ -1,3 +1,4 @@
+//go:build !linux
 // +build !linux
 
 package tproxy
@@ -5,7 +6,6 @@ package tproxy
 import (
 	"errors"
 	"github.com/tobyxdd/hysteria/pkg/core"
-	"github.com/tobyxdd/hysteria/pkg/transport"
 	"net"
 	"time"
 )
@@ -14,8 +14,8 @@ var ErrTimeout = errors.New("inactivity timeout")
 
 type UDPTProxy struct{}
 
-func NewUDPTProxy(hyClient *core.Client, transport transport.Transport, listen string, timeout time.Duration,
-	connFunc func(addr net.Addr), errorFunc func(addr net.Addr, err error)) (*UDPTProxy, error) {
+func NewUDPTProxy(hyClient *core.Client, listen string, timeout time.Duration,
+	connFunc func(addr, reqAddr net.Addr), errorFunc func(addr, reqAddr net.Addr, err error)) (*UDPTProxy, error) {
 	return nil, errors.New("not supported on the current system")
 }
 
